package solutions.bitbadger.documents.scala.query

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.CountQuery
import solutions.bitbadger.documents.scala.ClearConfiguration
import solutions.bitbadger.documents.support.ForceDialect
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

class CountQuerySpec extends AnyFunSpec with ClearConfiguration with Matchers {

  describe("all") {
    it("generates correctly") {
      CountQuery.all(TEST_TABLE) shouldEqual s"SELECT COUNT(*) AS it FROM $TEST_TABLE"
    }
  }

  describe("byFields") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      CountQuery.byFields(TEST_TABLE, List(Field.equal("test", "", ":field0")).asJava) shouldEqual
        s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      CountQuery.byFields(TEST_TABLE, List(Field.equal("test", "", ":field0")).asJava) shouldEqual
        s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data->>'test' = :field0"
    }
  }

  describe("byContains") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      CountQuery.byContains(TEST_TABLE) shouldEqual s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE data @> :criteria"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy CountQuery.byContains(TEST_TABLE)
    }
  }

  describe("byJsonPath") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      CountQuery.byJsonPath(TEST_TABLE) shouldEqual
        s"SELECT COUNT(*) AS it FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy CountQuery.byJsonPath(TEST_TABLE)
    }
  }
}
