package solutions.bitbadger.documents.scala.query

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.DeleteQuery
import solutions.bitbadger.documents.scala.ClearConfiguration
import solutions.bitbadger.documents.support.ForceDialect
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

class DeleteQuerySpec extends AnyFunSpec with ClearConfiguration with Matchers {

  describe("byId") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      DeleteQuery.byId(TEST_TABLE) shouldEqual s"DELETE FROM $TEST_TABLE WHERE data->>'id' = :id"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      DeleteQuery.byId(TEST_TABLE) shouldEqual s"DELETE FROM $TEST_TABLE WHERE data->>'id' = :id"
    }
  }

  describe("byFields") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      DeleteQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b")).asJava) shouldEqual
        s"DELETE FROM $TEST_TABLE WHERE data->>'a' = :b"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      DeleteQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b")).asJava) shouldEqual
        s"DELETE FROM $TEST_TABLE WHERE data->>'a' = :b"
    }
  }

  describe("byContains") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      DeleteQuery.byContains(TEST_TABLE) shouldEqual s"DELETE FROM $TEST_TABLE WHERE data @> :criteria"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy DeleteQuery.byContains(TEST_TABLE)
    }
  }

  describe("byJsonPath") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      DeleteQuery.byJsonPath(TEST_TABLE) shouldEqual
        s"DELETE FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy DeleteQuery.byJsonPath(TEST_TABLE)
    }
  }
}
