package solutions.bitbadger.documents.scala.query

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.ExistsQuery
import solutions.bitbadger.documents.scala.ClearConfiguration
import solutions.bitbadger.documents.support.ForceDialect
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

class ExistsQuerySpec extends AnyFunSpec with ClearConfiguration with Matchers {

  describe("byId") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      ExistsQuery.byId(TEST_TABLE) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      ExistsQuery.byId(TEST_TABLE) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'id' = :id) AS it"
    }
  }

  describe("byFields") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      ExistsQuery.byFields(TEST_TABLE, List(Field.equal("it", 7, ":test")).asJava) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE (data->>'it')::numeric = :test) AS it"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      ExistsQuery.byFields(TEST_TABLE, List(Field.equal("it", 7, ":test")).asJava) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data->>'it' = :test) AS it"
    }
  }

  describe("byContains") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      ExistsQuery.byContains(TEST_TABLE) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE data @> :criteria) AS it"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy ExistsQuery.byContains(TEST_TABLE)
    }
  }

  describe("byJsonPath") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      ExistsQuery.byJsonPath(TEST_TABLE) shouldEqual
        s"SELECT EXISTS (SELECT 1 FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)) AS it"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy ExistsQuery.byJsonPath(TEST_TABLE)
    }
  }
}
