package solutions.bitbadger.documents.scala.query

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.FindQuery
import solutions.bitbadger.documents.scala.ClearConfiguration
import solutions.bitbadger.documents.support.ForceDialect
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

class FindQuerySpec extends AnyFunSpec with ClearConfiguration with Matchers {

  describe("all") {
    it("generates correctly") {
      FindQuery.all(TEST_TABLE) shouldEqual s"SELECT data FROM $TEST_TABLE"
    }
  }

  describe("byId") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      FindQuery.byId(TEST_TABLE) shouldEqual s"SELECT data FROM $TEST_TABLE WHERE data->>'id' = :id"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      FindQuery.byId(TEST_TABLE) shouldEqual s"SELECT data FROM $TEST_TABLE WHERE data->>'id' = :id"
    }
  }

  describe("byFields") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      FindQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b"), Field.less("c", 14, ":d")).asJava) shouldEqual
        s"SELECT data FROM $TEST_TABLE WHERE data->>'a' = :b AND (data->>'c')::numeric < :d"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      FindQuery.byFields(TEST_TABLE, List(Field.equal("a", "", ":b"), Field.less("c", 14, ":d")).asJava) shouldEqual
        s"SELECT data FROM $TEST_TABLE WHERE data->>'a' = :b AND data->>'c' < :d"
    }
  }

  describe("byContains") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      FindQuery.byContains(TEST_TABLE) shouldEqual s"SELECT data FROM $TEST_TABLE WHERE data @> :criteria"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy FindQuery.byContains(TEST_TABLE)
    }
  }

  describe("byJsonPath") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      FindQuery.byJsonPath(TEST_TABLE) shouldEqual
        s"SELECT data FROM $TEST_TABLE WHERE jsonb_path_exists(data, :path::jsonpath)"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy FindQuery.byJsonPath(TEST_TABLE)
    }
  }
}
