package solutions.bitbadger.documents.scala.query

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.{DocumentException, Field}
import solutions.bitbadger.documents.query.PatchQuery
import solutions.bitbadger.documents.scala.ClearConfiguration
import solutions.bitbadger.documents.support.ForceDialect
import solutions.bitbadger.documents.support.TypesKt.TEST_TABLE

import scala.jdk.CollectionConverters.*

class PatchQuerySpec extends AnyFunSpec with ClearConfiguration with Matchers {

  describe("byId") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      PatchQuery.byId(TEST_TABLE) shouldEqual s"UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'id' = :id"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      PatchQuery.byId(TEST_TABLE) shouldEqual
        s"UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'id' = :id"
    }
  }

  describe("byFields") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      PatchQuery.byFields(TEST_TABLE, List(Field.equal("z", "", ":y")).asJava) shouldEqual
        s"UPDATE $TEST_TABLE SET data = data || :data WHERE data->>'z' = :y"
    }
    it("generates correctly | SQLite") {
      ForceDialect.sqlite()
      PatchQuery.byFields(TEST_TABLE, List(Field.equal("z", "", ":y")).asJava) shouldEqual
        s"UPDATE $TEST_TABLE SET data = json_patch(data, json(:data)) WHERE data->>'z' = :y"
    }
  }

  describe("byContains") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      PatchQuery.byContains(TEST_TABLE) shouldEqual
        s"UPDATE $TEST_TABLE SET data = data || :data WHERE data @> :criteria"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy PatchQuery.byContains(TEST_TABLE)
    }
  }

  describe("byJsonPath") {
    it("generates correctly | PostgreSQL") {
      ForceDialect.postgres()
      PatchQuery.byJsonPath(TEST_TABLE) shouldEqual
        s"UPDATE $TEST_TABLE SET data = data || :data WHERE jsonb_path_exists(data, :path::jsonpath)"
    }
    it("fails | SQLite") {
      ForceDialect.sqlite()
      a [DocumentException] should be thrownBy PatchQuery.byJsonPath(TEST_TABLE)
    }
  }
}
